@@ -76,11 +76,6 @@
 #define DISCONNECT_TIMEOUT 5000
 #define STREAM_TIMEOUT 20000
 
-typedef enum {
-	AVDTP_ERROR_ERRNO,
-	AVDTP_ERROR_ERROR_CODE
-} avdtp_error_type_t;
-
 typedef enum {
 	AVDTP_SESSION_STATE_DISCONNECTED,
 	AVDTP_SESSION_STATE_CONNECTING,
@@ -362,14 +357,6 @@ struct avdtp {
 	DBusPendingCall *pending_auth;
 };
 
-struct avdtp_error {
-	avdtp_error_type_t type;
-	union {
-		uint8_t error_code;
-		int posix_errno;
-	} err;
-};
-
 static uint8_t free_seid = 1;
 static GSList *local_seps = NULL;
 
@@ -499,7 +486,7 @@ static void set_disconnect_timer(struct avdtp *session)
 						disconnect_timeout, session);
 }
 
-static void avdtp_error_init(struct avdtp_error *err, uint8_t type, int id)
+void avdtp_error_init(struct avdtp_error *err, uint8_t type, int id)
 {
 	err->type = type;
 	switch (type) {
@@ -685,6 +672,8 @@ static void avdtp_sep_set_state(struct avdtp *session,
 		session->streams = g_slist_remove(session->streams, stream);
 		if (session->pending_open == stream)
 			handle_transport_connect(session, -1, 0);
+		if (session->req && session->req->stream == stream)
+			session->req->stream = NULL;
 		stream_free(stream);
 		if (session->ref == 1 && !session->streams)
 			set_disconnect_timer(session);
@@ -696,10 +685,15 @@ static void avdtp_sep_set_state(struct avdtp *session,
 
 static void finalize_discovery(struct avdtp *session, int err)
 {
+	struct avdtp_error avdtp_err;
+
+	avdtp_error_init(&avdtp_err, AVDTP_ERROR_ERRNO, -err);
+
 	if (!session->discov_cb)
 		return;
 
-	session->discov_cb(session, session->seps, err,
+	session->discov_cb(session, session->seps,
+				err ? &avdtp_err : NULL,
 				session->user_data);
 
 	session->discov_cb = NULL;
@@ -1508,11 +1502,6 @@ static gboolean l2cap_connect_cb(GIOChannel *chan, GIOCondition cond,
 
 	sk = g_io_channel_unix_get_fd(chan);
 
-	if (cond & (G_IO_ERR | G_IO_HUP)) {
-		err = EIO;
-		goto failed;
-	}
-
 	len = sizeof(ret);
 	if (getsockopt(sk, SOL_SOCKET, SO_ERROR, &ret, &len) < 0) {
 		err = errno;
@@ -1526,6 +1515,11 @@ static gboolean l2cap_connect_cb(GIOChannel *chan, GIOCondition cond,
 		goto failed;
 	}
 
+	if (cond & G_IO_HUP) {
+		err = EIO;
+		goto failed;
+	}
+
 	ba2str(&session->dst, address);
 	debug("AVDTP: connected %s channel to %s",
 			session->pending_open ? "transport" : "signaling",
