@@ -122,6 +122,11 @@ static struct device *create_device(bdaddr_t *bda)
 	return device_register(connection, path, bda);
 }
 
+static void destroy_device(struct device *device)
+{
+	dbus_connection_destroy_object_path(connection, device->path);
+}
+
 static void remove_device(struct device *device)
 {
 	if (device == default_dev) {
@@ -136,7 +141,7 @@ static void remove_device(struct device *device)
 
 	devices = g_slist_remove(devices, device);
 
-	dbus_connection_destroy_object_path(connection, device->path);
+	destroy_device(device);
 }
 
 static gboolean add_device(struct device *device, gboolean send_signals)
@@ -309,6 +314,7 @@ static void finish_sdp(struct audio_sdp_data *data, gboolean success)
 	if (dbus_error_is_set(&derr)) {
 		error("Unable to get message args");
 		success = FALSE;
+		err_failed(connection, data->msg, derr.message);
 		dbus_error_free(&derr);
 		goto done;
 	}
@@ -346,8 +352,10 @@ done:
 		if (data->cb)
 			data->cb(data->device, data->cb_data);
 	} else {
-		remove_device(data->device);
-		data->cb(NULL, data->cb_data);
+		if (data->cb)
+			data->cb(NULL, data->cb_data);
+		if (!g_slist_find(devices, data->device))
+			destroy_device(data->device);
 	}
 	if (data->msg)
 		dbus_message_unref(data->msg);
@@ -611,7 +619,7 @@ struct device *manager_device_connected(bdaddr_t *bda, const char *uuid)
 		if (!device)
 			return NULL;
 		if (!add_device(device, TRUE)) {
-			remove_device(device);
+			destroy_device(device);
 			return NULL;
 		}
 		created = TRUE;
