@@ -248,7 +248,20 @@ const char *mode2str(uint8_t mode)
 	}
 }
 
-uint8_t str2mode(const char *mode)
+static uint8_t on_mode(const char *addr)
+{
+	char mode[14];
+	bdaddr_t sba;
+
+	str2ba(addr, &sba);
+
+	if (read_on_mode(&sba, mode, sizeof(mode)) < 0)
+		return MODE_CONNECTABLE;
+
+	return str2mode(addr, mode);
+}
+
+uint8_t str2mode(const char *addr, const char *mode)
 {
 	if (strcasecmp("off", mode) == 0)
 		return MODE_OFF;
@@ -258,6 +271,8 @@ uint8_t str2mode(const char *mode)
 		return MODE_DISCOVERABLE;
 	else if (strcasecmp("limited", mode) == 0)
 		return MODE_LIMITED;
+	else if (strcasecmp("on", mode) == 0)
+		return on_mode(addr);
 	else
 		return MODE_UNKNOWN;
 }
@@ -531,7 +546,7 @@ static DBusHandlerResult adapter_set_mode(DBusConnection *conn,
 	if (!mode)
 		return error_invalid_arguments(conn, msg);
 
-	new_mode = str2mode(mode);
+	new_mode = str2mode(adapter->address, mode);
 	switch(new_mode) {
 	case MODE_OFF:
 		scan_enable = SCAN_DISABLED;
@@ -547,6 +562,9 @@ static DBusHandlerResult adapter_set_mode(DBusConnection *conn,
 		return error_invalid_arguments(conn, msg);
 	}
 
+	/* Do reverse resolution in case of "on" mode */
+	mode = mode2str(new_mode);
+
 	dd = hci_open_dev(adapter->dev_id);
 	if (dd < 0)
 		return error_no_such_adapter(conn, msg);
@@ -1083,9 +1101,6 @@ static DBusHandlerResult adapter_set_name(DBusConnection *conn,
 	char *str_ptr;
 	int ecode;
 
-	if (!adapter->up)
-		return error_not_ready(conn, msg);
-
 	if (!dbus_message_get_args(msg, NULL,
 				DBUS_TYPE_STRING, &str_ptr,
 				DBUS_TYPE_INVALID))
@@ -1096,14 +1111,18 @@ static DBusHandlerResult adapter_set_name(DBusConnection *conn,
 		return error_invalid_arguments(conn, msg);
 	}
 
-	hci_devba(adapter->dev_id, &bdaddr);
+	str2ba(adapter->address, &bdaddr);
 
 	write_local_name(&bdaddr, str_ptr);
 
+	if (!adapter->up)
+		goto done;
+
 	ecode = set_device_name(adapter->dev_id, str_ptr);
 	if (ecode < 0)
 		return error_failed(conn, msg, -ecode);
 
+done:
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
@@ -3281,7 +3300,7 @@ GSList *service_classes_str(uint32_t class)
 
 static DBusMethodVTable adapter_methods[] = {
 	{ "GetInfo",				adapter_get_info,
-		"",	"{sv}"	},
+		"",	"a{sv}"	},
 	{ "GetAddress",				adapter_get_address,
 		"",	"s"	},
 	{ "GetVersion",				adapter_get_version,
@@ -3326,7 +3345,7 @@ static DBusMethodVTable adapter_methods[] = {
 		"s",	""	},
 
 	{ "GetRemoteInfo",			adapter_get_remote_info,
-		"s",	"{sv}"	},
+		"s",	"a{sv}"	},
 	{ "GetRemoteServiceRecord",		adapter_get_remote_svc,
 		"su",	"ay"	},
 	{ "GetRemoteServiceRecordAsXML",	adapter_get_remote_svc_xml,
