@@ -29,7 +29,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <string.h>
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -39,11 +38,21 @@
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
-#include "hcid.h"
+#include <glib.h>
 
+#include <dbus/dbus.h>
+
+#include "dbus-helper.h"
+
+#include "hcid.h"
+#include "sdpd.h"
+
+#include "logging.h"
 #include "textfile.h"
 #include "oui.h"
 
+#include "device.h"
+
 #define MAX_DEVICES 16
 
 struct hci_peer {
@@ -81,7 +90,9 @@ struct hci_dev {
 	uint16_t hci_rev;
 	uint16_t manufacturer;
 
+	uint8_t  ssp_mode;
 	uint8_t  name[248];
+	uint8_t  class[3];
 
 	struct hci_peer *peers;
 	struct hci_conn *conns;
@@ -91,7 +102,7 @@ static struct hci_dev devices[MAX_DEVICES];
 
 #define ASSERT_DEV_ID { if (dev_id >= MAX_DEVICES) return -ERANGE; }
 
-void init_devices(void)
+void init_adapters(void)
 {
 	int i;
 
@@ -106,14 +117,14 @@ static int device_read_bdaddr(uint16_t dev_id, bdaddr_t *bdaddr)
 	dd = hci_open_dev(dev_id);
 	if (dd < 0) {
 		error("Can't open device hci%d",
-		      dev_id, strerror(errno), errno);
+					dev_id, strerror(errno), errno);
 		return -errno;
 	}
 
 	if (hci_read_bd_addr(dd, bdaddr, 2000) < 0) {
 		int err = errno;
 		error("Can't read address for hci%d: %s (%d)",
-		      dev_id, strerror(errno), errno);
+					dev_id, strerror(errno), errno);
 		hci_close_dev(dd);
 		return -err;
 	}
@@ -123,7 +134,7 @@ static int device_read_bdaddr(uint16_t dev_id, bdaddr_t *bdaddr)
 	return 0;
 }
 
-int add_device(uint16_t dev_id)
+int add_adapter(uint16_t dev_id)
 {
 	struct hci_dev *dev;
 	struct hci_dev_info di;
@@ -145,9 +156,9 @@ int add_device(uint16_t dev_id)
 	if (bacmp(&di.bdaddr, BDADDR_ANY))
 		bacpy(&dev->bdaddr, &di.bdaddr);
 	else {
-		int ret = device_read_bdaddr(dev_id, &dev->bdaddr);
-		if (ret < 0)
-			return ret;
+		int err = device_read_bdaddr(dev_id, &dev->bdaddr);
+		if (err < 0)
+			return err;
 	}
 	memcpy(dev->features, di.features, 8);
 
@@ -156,7 +167,7 @@ int add_device(uint16_t dev_id)
 	return 0;
 }
 
-int remove_device(uint16_t dev_id)
+int remove_adapter(uint16_t dev_id)
 {
 	struct hci_dev *dev;
 
@@ -199,12 +210,31 @@ static inline uint8_t get_inquiry_mode(struct hci_dev *dev)
 	return 0;
 }
 
-int start_device(uint16_t dev_id)
+static void update_ext_inquiry_response(int dd, struct hci_dev *dev)
+{
+	uint8_t fec = 0, data[240];
+
+	if (!(dev->features[6] & LMP_EXT_INQ))
+		return;
+
+	memset(data, 0, sizeof(data));
+
+	if (dev->ssp_mode > 0)
+		create_ext_inquiry_response((char *) dev->name, data);
+
+	if (hci_write_ext_inquiry_response(dd, fec, data, 2000) < 0)
+		error("Can't write extended inquiry response: %s (%d)",
+						strerror(errno), errno);
+}
+
+int start_adapter(uint16_t dev_id)
 {
 	struct hci_dev *dev;
 	struct hci_version ver;
 	uint8_t features[8], inqmode;
-	int dd;
+	uint8_t events[8] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x00, 0x00 };
+	char name[249];
+	int dd, err;
 
 	ASSERT_DEV_ID;
 
@@ -234,15 +264,84 @@ int start_device(uint16_t dev_id)
 	dev->manufacturer = ver.manufacturer;
 
 	if (hci_read_local_features(dd, features, 1000) < 0) {
-		int err = errno;
+		err = errno;
 		error("Can't read features for hci%d: %s (%d)",
-					dev_id, strerror(errno), errno);
+					dev_id, strerror(err), err);
 		hci_close_dev(dd);
 		return -err;
 	}
 
 	memcpy(dev->features, features, 8);
 
+	if (hci_read_class_of_dev(dd, dev->class, 1000) < 0) {
+		err = errno;
+		error("Can't read class of device on hci%d: %s (%d)",
+						dev_id, strerror(err), err);
+		hci_close_dev(dd);
+		return -err;
+	}
+
+	if (hci_read_local_name(dd, sizeof(name), name, 2000) < 0) {
+		err = errno;
+		error("Can't read local name on hci%d: %s (%d)",
+						dev_id, strerror(err), err);
+		hci_close_dev(dd);
+		return -err;
+	}
+
+	memcpy(dev->name, name, 248);
+
+	if (!(features[6] & LMP_SIMPLE_PAIR))
+		goto setup;
+
+	if (hci_read_simple_pairing_mode(dd, &dev->ssp_mode, 1000) < 0) {
+		err = errno;
+		error("Can't read simple pairing mode on hci%d: %s (%d)",
+						dev_id, strerror(err), err);
+		hci_close_dev(dd);
+		return -err;
+	}
+
+setup:
+	if (ver.hci_rev > 1) {
+		if (features[5] & LMP_SNIFF_SUBR)
+			events[5] |= 0x20;
+
+		if (features[5] & LMP_PAUSE_ENC)
+			events[5] |= 0x80;
+
+		if (features[6] & LMP_EXT_INQ)
+			events[5] |= 0x40;
+
+		if (features[6] & LMP_NFLUSH_PKTS)
+			events[7] |= 0x01;
+			
+		if (features[7] & LMP_LSTO)
+			events[6] |= 0x80;
+
+		if (features[6] & LMP_SIMPLE_PAIR) {
+			events[6] |= 0x01;	/* IO Capability Request */
+			events[6] |= 0x02;	/* IO Capability Response */
+			events[6] |= 0x04;	/* User Confirmation Request */
+			events[6] |= 0x08;	/* User Passkey Request */
+			events[6] |= 0x10;	/* Remote OOB Data Request */
+			events[6] |= 0x20;	/* Simple Pairing Complete */
+			events[7] |= 0x04;	/* User Passkey Notification */
+			events[7] |= 0x08;	/* Keypress Notification */
+			events[7] |= 0x10;	/* Remote Host Supported Features Notification */
+		}
+
+		hci_send_cmd(dd, OGF_HOST_CTL, OCF_SET_EVENT_MASK,
+						sizeof(events), events);
+	}
+
+	if (read_local_name(&dev->bdaddr, name) == 0) {
+		memcpy(dev->name, name, 248);
+		hci_write_local_name(dd, name, 5000);
+        }
+
+	update_ext_inquiry_response(dd, dev);
+
 	inqmode = get_inquiry_mode(dev);
 	if (inqmode < 1)
 		goto done;
@@ -250,7 +349,7 @@ int start_device(uint16_t dev_id)
 	if (hci_write_inquiry_mode(dd, inqmode, 1000) < 0) {
 		int err = errno;
 		error("Can't write inquiry mode for hci%d: %s (%d)",
-					dev_id, strerror(errno), errno);
+						dev_id, strerror(errno), err);
 		hci_close_dev(dd);
 		return -err;
 	}
@@ -263,7 +362,7 @@ done:
 	return 0;
 }
 
-int stop_device(uint16_t dev_id)
+int stop_adapter(uint16_t dev_id)
 {
 	ASSERT_DEV_ID;
 
@@ -272,6 +371,32 @@ int stop_device(uint16_t dev_id)
 	return 0;
 }
 
+int update_adapter(uint16_t dev_id)
+{
+	struct hci_dev *dev;
+	int dd;
+
+	ASSERT_DEV_ID;
+
+	dev = &devices[dev_id];
+
+	if (dev->ignore)
+		return 0;
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		error("Can't open device hci%d",
+					dev_id, strerror(errno), errno);
+		return -errno;
+	}
+
+	update_ext_inquiry_response(dd, dev);
+
+	hci_close_dev(dd);
+
+	return 0;
+}
+
 int get_device_address(uint16_t dev_id, char *address, size_t size)
 {
 	struct hci_dev *dev;
@@ -286,6 +411,29 @@ int get_device_address(uint16_t dev_id, char *address, size_t size)
 	return ba2str(&dev->bdaddr, address);
 }
 
+int get_device_class(uint16_t dev_id, uint8_t *cls)
+{
+	struct hci_dev *dev;
+
+	ASSERT_DEV_ID;
+
+	dev = &devices[dev_id];
+	memcpy(cls, dev->class, 3);
+
+	return 0;
+}
+
+int set_device_class(uint16_t dev_id, uint8_t *cls)
+{
+	struct hci_dev *dev;
+
+	ASSERT_DEV_ID;
+	dev = &devices[dev_id];
+	memcpy(dev->class, cls, 3);
+
+	return 0;
+}
+
 int get_device_version(uint16_t dev_id, char *version, size_t size)
 {
 	struct hci_dev *dev;
@@ -299,7 +447,7 @@ int get_device_version(uint16_t dev_id, char *version, size_t size)
 
 	dev = &devices[dev_id];
 
-	if (dev->lmp_ver == 0x03 &&
+	if ((dev->lmp_ver == 0x03 || dev->lmp_ver == 0x04) &&
 			(dev->features[3] & (LMP_EDR_ACL_2M | LMP_EDR_ACL_3M)))
 		sprintf(edr, " + EDR");
 	else
@@ -409,6 +557,31 @@ int get_device_company(uint16_t dev_id, char *company, size_t size)
 	return err;
 }
 
+int set_simple_pairing_mode(uint16_t dev_id, uint8_t mode)
+{
+	struct hci_dev *dev;
+	int dd;
+
+	ASSERT_DEV_ID;
+
+	dev = &devices[dev_id];
+
+	dev->ssp_mode = mode;
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0) {
+		error("Can't open device hci%d",
+					dev_id, strerror(errno), errno);
+		return -errno;
+	}
+
+	update_ext_inquiry_response(dd, dev);
+
+	hci_close_dev(dd);
+
+	return 0;
+}
+
 int get_device_name(uint16_t dev_id, char *name, size_t size)
 {
 	char tmp[249];
@@ -464,28 +637,9 @@ int set_device_name(uint16_t dev_id, const char *name)
 		return -err;
 	}
 
-	if (dev->features[6] & LMP_EXT_INQ) {
-		uint8_t fec = 0, data[240];
-		int len;
+	memcpy(dev->name, name, 248);
 
-		memset(data, 0, sizeof(data));
-		len = strlen(name);
-		if (len > 48) {
-			len = 48;
-			data[1] = 0x08;
-		} else
-			data[1] = 0x09;
-		data[0] = len + 1;
-		memcpy(data + 2, name, len);
-
-		if (hci_write_ext_inquiry_response(dd, fec, data, 2000) < 0) {
-			int err = errno;
-			error("Can't write extended inquiry response for hci%d: %s (%d)",
-							dev_id, strerror(errno), errno);
-			hci_close_dev(dd);
-			return -err;
-		}
-	}
+	update_ext_inquiry_response(dd, dev);
 
 	hci_close_dev(dd);
 
@@ -548,3 +702,82 @@ int get_encryption_key_size(uint16_t dev_id, const bdaddr_t *baddr)
 
 	return size;
 }
+
+static DBusConnection *connection = NULL;
+
+static GSList *device_list = NULL;
+
+gboolean device_init(DBusConnection *conn)
+{
+	connection = dbus_connection_ref(conn);
+	if (connection == NULL)
+		return FALSE;
+
+	return TRUE;
+}
+
+static void device_destroy(struct device_data *device)
+{
+	debug("Removing device %s", device->path);
+
+	dbus_connection_destroy_object_path(connection, device->path);
+}
+
+void device_cleanup(void)
+{
+	g_slist_foreach(device_list, (GFunc) device_destroy, NULL);
+	g_slist_free(device_list);
+
+	if (connection == NULL)
+		return;
+
+	dbus_connection_unref(connection);
+}
+
+void device_foreach(GFunc func, gpointer user_data)
+{
+	g_slist_foreach(device_list, func, user_data);
+}
+
+static void device_free(struct device_data *device)
+{
+	g_free(device->path);
+	g_free(device);
+}
+
+static void device_unregister(DBusConnection *conn, void *user_data)
+{
+	struct device_data *device = user_data;
+
+	device_list = g_slist_remove(device_list, device);
+
+	device_free(device);
+}
+
+struct device_data *device_create(const char *adapter, const char *address)
+{
+	struct device_data *device;
+
+	device = g_try_malloc0(sizeof(struct device_data));
+	if (device == NULL)
+		return NULL;
+
+	device->path = g_strdup_printf("/device/%s_%s", adapter, address);
+	g_strdelimit(device->path, ":", '_');
+
+	debug("Creating device %s", device->path);
+
+	if (dbus_connection_create_object_path(connection, device->path,
+					device, device_unregister) == FALSE) {
+		device_free(device);
+		return NULL;
+	}
+
+	device_list = g_slist_append(device_list, device);
+
+	return device;
+}
+
+void device_remove(const char *path)
+{
+}
