@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <string.h>
 #include <signal.h>
 #include <time.h>
@@ -46,11 +45,14 @@
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
 
-#include "glib-ectomy.h"
+#include <glib.h>
+
+#include <dbus/dbus.h>
 
 #include "hcid.h"
 #include "textfile.h"
-#include "list.h"
+#include "adapter.h"
+#include "dbus-hci.h"
 
 struct g_io_info {
 	GIOChannel	*channel;
@@ -62,24 +64,15 @@ static struct g_io_info io_data[HCI_MAX_DEV];
 
 static int pairing = HCID_PAIRING_MULTI;
 
-static struct slist *hci_req_queue = NULL;
+static GSList *hci_req_queue = NULL;
 
 struct hci_req_data *hci_req_data_new(int dev_id, const bdaddr_t *dba, uint16_t ogf, uint16_t ocf, int event, const void *cparam, int clen)
 {
 	struct hci_req_data *data;
 
-	data = malloc(sizeof(*data));
-	if (!data)
-		return NULL;
-
-	memset(data, 0, sizeof(*data));
-
-	data->cparam = malloc(clen);
-	if (!data->cparam) {
-		free(data);
-		return NULL;
-	}
+	data = g_new0(struct hci_req_data, 1);
 
+	data->cparam = g_malloc(clen);
 	memcpy(data->cparam, cparam, clen);
 
 	bacpy(&data->dba, dba);
@@ -110,7 +103,7 @@ static void hci_req_queue_process(int dev_id)
 	dd = hci_open_dev(dev_id);
 	do {
 		struct hci_req_data *data;
-		struct slist *l = slist_find(hci_req_queue, &dev_id, hci_req_find_by_devid);
+		GSList *l = g_slist_find_custom(hci_req_queue, &dev_id, hci_req_find_by_devid);
 
 		if (!l)
 			break;
@@ -120,9 +113,9 @@ static void hci_req_queue_process(int dev_id)
 		
 		ret_val = hci_send_cmd(dd, data->ogf, data->ocf, data->clen, data->cparam);
 		if (ret_val < 0) {
-			hci_req_queue = slist_remove(hci_req_queue, data);
-			free(data->cparam);
-			free(data);
+			hci_req_queue = g_slist_remove(hci_req_queue, data);
+			g_free(data->cparam);
+			g_free(data);
 		}
 
 	} while(ret_val < 0);
@@ -132,13 +125,13 @@ static void hci_req_queue_process(int dev_id)
 
 void hci_req_queue_append(struct hci_req_data *data)
 {
-	struct slist *l;
+	GSList *l;
 	struct hci_req_data *match;
 
 
-	hci_req_queue = slist_append(hci_req_queue, data);
+	hci_req_queue = g_slist_append(hci_req_queue, data);
 
-	l = slist_find(hci_req_queue, &data->dev_id, hci_req_find_by_devid);
+	l = g_slist_find_custom(hci_req_queue, &data->dev_id, hci_req_find_by_devid);
 	match = l->data;
 
 	if (match->status == REQ_SENT)
@@ -149,7 +142,7 @@ void hci_req_queue_append(struct hci_req_data *data)
 
 void hci_req_queue_remove(int dev_id, bdaddr_t *dba)
 {
-	struct slist *cur, *next;
+	GSList *cur, *next;
 	struct hci_req_data *req;
 
 	for (cur = hci_req_queue; cur != NULL; cur = next) {
@@ -158,22 +151,22 @@ void hci_req_queue_remove(int dev_id, bdaddr_t *dba)
 		if ((req->dev_id != dev_id) || (bacmp(&req->dba, dba)))
 			continue;
 
-		hci_req_queue = slist_remove(hci_req_queue, req);
-		free(req->cparam);
-		free(req);
+		hci_req_queue = g_slist_remove(hci_req_queue, req);
+		g_free(req->cparam);
+		g_free(req);
 	}
 }
 
 static void check_pending_hci_req(int dev_id, int event)
 {
 	struct hci_req_data *data;
-	struct slist *l;
+	GSList *l;
 
 	if (!hci_req_queue)
 		return;
 
 	/* find the first element(pending)*/
-	l = slist_find(hci_req_queue, &dev_id, hci_req_find_by_devid);
+	l = g_slist_find_custom(hci_req_queue, &dev_id, hci_req_find_by_devid);
 
 	if (!l)
 		return;
@@ -186,9 +179,9 @@ static void check_pending_hci_req(int dev_id, int event)
 			return;
 
 		/* remove the confirmed cmd */
-		hci_req_queue = slist_remove(hci_req_queue, data);
-		free(data->cparam);
-		free(data);
+		hci_req_queue = g_slist_remove(hci_req_queue, data);
+		g_free(data->cparam);
+		g_free(data);
 	}
 
 	hci_req_queue_process(dev_id);
@@ -201,11 +194,7 @@ static inline int get_bdaddr(int dev, bdaddr_t *sba, uint16_t handle, bdaddr_t *
 	char addr[18];
 	int i;
 
-	cl = malloc(10 * sizeof(*ci) + sizeof(*cl));
-	if (!cl)
-		return -ENOMEM;
-
-	memset(cl, 0, 10 * sizeof(*ci) + sizeof(*cl));
+	cl = g_malloc0(10 * sizeof(*ci) + sizeof(*cl));
 
 	ba2str(sba, addr);
 	cl->dev_id = hci_devid(addr);
@@ -213,18 +202,18 @@ static inline int get_bdaddr(int dev, bdaddr_t *sba, uint16_t handle, bdaddr_t *
 	ci = cl->conn_info;
 
 	if (ioctl(dev, HCIGETCONNLIST, (void *) cl) < 0) {
-		free(cl);
+		g_free(cl);
 		return -EIO;
 	}
 
 	for (i = 0; i < cl->conn_num; i++, ci++)
 		if (ci->handle == handle) {
 			bacpy(dba, &ci->bdaddr);
-			free(cl);
+			g_free(cl);
 			return 0;
 		}
 
-	free(cl);
+	g_free(cl);
 
 	return -ENOENT;
 }
@@ -318,6 +307,39 @@ static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
 	}
 }
 
+/* Simple Pairing handling */
+
+static void user_confirm_request(int dev, bdaddr_t *sba, void *ptr)
+{
+	hci_send_cmd(dev, OGF_LINK_CTL, OCF_USER_CONFIRM_NEG_REPLY, 6, ptr);
+}
+
+static void user_passkey_request(int dev, bdaddr_t *sba, void *ptr)
+{
+	hci_send_cmd(dev, OGF_LINK_CTL, OCF_USER_PASSKEY_NEG_REPLY, 6, ptr);
+}
+
+static void remote_oob_data_request(int dev, bdaddr_t *sba, void *ptr)
+{
+	hci_send_cmd(dev, OGF_LINK_CTL, OCF_REMOTE_OOB_DATA_NEG_REPLY, 6, ptr);
+}
+
+static void io_capa_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
+{
+	io_capability_neg_reply_cp cp;
+	char sa[18], da[18];
+
+	ba2str(sba, sa); ba2str(dba, da);
+	info("io_capa_request (sba=%s, dba=%s)", sa, da);
+
+	memset(&cp, 0, sizeof(cp));
+	bacpy(&cp.bdaddr, dba);
+	cp.reason = HCI_PAIRING_NOT_ALLOWED;
+
+	hci_send_cmd(dev, OGF_LINK_CTL, OCF_IO_CAPABILITY_NEG_REPLY,
+					IO_CAPABILITY_NEG_REPLY_CP_SIZE, &cp);
+}
+
 /* PIN code handling */
 
 void set_pin_length(bdaddr_t *sba, int length)
@@ -346,11 +368,7 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	ba2str(sba, sa); ba2str(dba, da);
 	info("pin_code_request (sba=%s, dba=%s)", sa, da);
 
-	cr = malloc(sizeof(*cr) + sizeof(*ci));
-	if (!cr)
-		return;
-
-	memset(cr, 0, sizeof(*cr) + sizeof(*ci));
+	cr = g_malloc0(sizeof(*cr) + sizeof(*ci));
 
 	bacpy(&cr->bdaddr, dba);
 	cr->type = ACL_LINK;
@@ -390,23 +408,28 @@ static void pin_code_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 				hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_REPLY,
 					PIN_CODE_REPLY_CP_SIZE, &pr);
 			} else {
-				/* Let PIN helper handle that */ 
+				/* Request PIN from passkey agent */
 				hcid_dbus_request_pin(dev, sba, ci);
 			}
 		}
 	} else {
-		/* Let PIN helper handle that */ 
-		hcid_dbus_request_pin(dev, sba, ci);
+		if (pinlen > 0) {
+			/* Confirm PIN by passkey agent */
+			hcid_dbus_confirm_pin(dev, sba, ci, pin);
+		} else {
+			/* Request PIN from passkey agent */ 
+			hcid_dbus_request_pin(dev, sba, ci);
+		}
 	}
 
 	hcid_dbus_pending_pin_req_add(sba, &ci->bdaddr);
 
-	free(cr);
+	g_free(cr);
 
 	return;
 
 reject:
-	free(cr);
+	g_free(cr);
 
 	hci_send_cmd(dev, OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY, 6, dba);
 
@@ -449,6 +472,12 @@ static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_SCAN_ENABLE):
 		hcid_dbus_setscan_enable_complete(sba);
 		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_CLASS_OF_DEV):
+		hcid_dbus_write_class_complete(sba);
+		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_SIMPLE_PAIRING_MODE):
+		hcid_dbus_write_simple_pairing_mode_complete(sba);
+		break;
 	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PIN_CODE_REPLY):
 	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PIN_CODE_NEG_REPLY):
 		hcid_dbus_pin_code_reply(sba, ptr);
@@ -595,9 +624,11 @@ static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 	evt_conn_complete *evt = ptr;
 	char filename[PATH_MAX];
 	remote_name_req_cp cp_name;
-	bdaddr_t tmp;
 	struct hci_req_data *data;
-	char *str, *local_addr, *peer_addr;
+	char local_addr[18], peer_addr[18], *str;
+
+	if (evt->link_type != ACL_LINK)
+		return;
 
 	hcid_dbus_conn_complete(sba, evt->status, evt->handle, &evt->bdaddr);
 
@@ -618,8 +649,8 @@ static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 	hci_req_queue_append(data);
 
 	/* check if the remote version needs be requested */
-	baswap(&tmp, sba); local_addr = batostr(&tmp);
-	baswap(&tmp, &evt->bdaddr); peer_addr = batostr(&tmp);
+	ba2str(sba, local_addr);
+	ba2str(&evt->bdaddr, peer_addr);
 
 	create_name(filename, sizeof(filename), STORAGEDIR, local_addr, "manufacturers");
 
@@ -636,9 +667,6 @@ static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 		hci_req_queue_append(data);
 	} else
 		free(str);
-
-	free(local_addr);
-	free(peer_addr);
 }
 
 static inline void disconn_complete(int dev, bdaddr_t *sba, void *ptr)
@@ -783,6 +811,22 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 	case EVT_RETURN_LINK_KEYS:
 		return_link_keys(dev, &di->bdaddr, ptr);
 		break;
+
+	case EVT_IO_CAPABILITY_REQUEST:
+		io_capa_request(dev, &di->bdaddr, (bdaddr_t *) ptr);
+		break;
+
+	case EVT_USER_CONFIRM_REQUEST:
+		user_confirm_request(dev, &di->bdaddr, ptr);
+		break;
+
+	case EVT_USER_PASSKEY_REQUEST:
+		user_passkey_request(dev, &di->bdaddr, ptr);
+		break;
+
+	case EVT_REMOTE_OOB_DATA_REQUEST:
+		remote_oob_data_request(dev, &di->bdaddr, ptr);
+		break;
 	}
 
 	return TRUE;
@@ -816,17 +860,26 @@ void start_security_manager(int hdev)
 	hci_filter_set_event(EVT_LINK_KEY_REQ, &flt);
 	hci_filter_set_event(EVT_LINK_KEY_NOTIFY, &flt);
 	hci_filter_set_event(EVT_RETURN_LINK_KEYS, &flt);
+	hci_filter_set_event(EVT_IO_CAPABILITY_REQUEST, &flt);
+	hci_filter_set_event(EVT_IO_CAPABILITY_RESPONSE, &flt);
+	hci_filter_set_event(EVT_USER_CONFIRM_REQUEST, &flt);
+	hci_filter_set_event(EVT_USER_PASSKEY_REQUEST, &flt);
+	hci_filter_set_event(EVT_REMOTE_OOB_DATA_REQUEST, &flt);
+	hci_filter_set_event(EVT_USER_PASSKEY_NOTIFY, &flt);
+	hci_filter_set_event(EVT_KEYPRESS_NOTIFY, &flt);
+	hci_filter_set_event(EVT_SIMPLE_PAIRING_COMPLETE, &flt);
+	hci_filter_set_event(EVT_AUTH_COMPLETE, &flt);
 	hci_filter_set_event(EVT_REMOTE_NAME_REQ_COMPLETE, &flt);
 	hci_filter_set_event(EVT_READ_REMOTE_VERSION_COMPLETE, &flt);
 	hci_filter_set_event(EVT_READ_REMOTE_FEATURES_COMPLETE, &flt);
+	hci_filter_set_event(EVT_REMOTE_HOST_FEATURES_NOTIFY, &flt);
 	hci_filter_set_event(EVT_INQUIRY_COMPLETE, &flt);
 	hci_filter_set_event(EVT_INQUIRY_RESULT, &flt);
 	hci_filter_set_event(EVT_INQUIRY_RESULT_WITH_RSSI, &flt);
 	hci_filter_set_event(EVT_EXTENDED_INQUIRY_RESULT, &flt);
+	hci_filter_set_event(EVT_CONN_REQUEST, &flt);
 	hci_filter_set_event(EVT_CONN_COMPLETE, &flt);
 	hci_filter_set_event(EVT_DISCONN_COMPLETE, &flt);
-	hci_filter_set_event(EVT_AUTH_COMPLETE, &flt);
-	hci_filter_set_event(EVT_CONN_REQUEST, &flt);
 	if (setsockopt(dev, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
 		error("Can't set filter on hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
@@ -834,19 +887,12 @@ void start_security_manager(int hdev)
 		return;
 	}
 
-	di = malloc(sizeof(*di));
-	if (!di) {
-		error("Can't allocate device info buffer: %s (%d)",
-							strerror(errno), errno);
-		close(dev);
-		return;
-	}
-
+	di = g_new(struct hci_dev_info, 1);
 	if (hci_devinfo(hdev, di) < 0) {
 		error("Can't get device info: %s (%d)",
 							strerror(errno), errno);
 		close(dev);
-		free(di);
+		g_free(di);
 		return;
 	}
 
@@ -854,7 +900,7 @@ void start_security_manager(int hdev)
 	g_io_channel_set_close_on_unref(chan, TRUE);
 	io_data[hdev].watch_id = g_io_add_watch_full(chan, G_PRIORITY_HIGH,
 						G_IO_IN | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
-						io_security_event, di, (GDestroyNotify)free);
+						io_security_event, di, (GDestroyNotify) g_free);
 	io_data[hdev].channel = chan;
 	io_data[hdev].pin_length = -1;
 
@@ -877,7 +923,7 @@ void stop_security_manager(int hdev)
 
 	info("Stopping security manager %d", hdev);
 
-	g_io_remove_watch(io_data[hdev].watch_id);
+	g_source_remove(io_data[hdev].watch_id);
 	g_io_channel_unref(io_data[hdev].channel);
 	io_data[hdev].watch_id = -1;
 	io_data[hdev].channel = NULL;
