@@ -21,10 +21,25 @@
  *
  */
 
+#include "connection.h"
+#include "server.h"
+
 #define MAX_PATH_LENGTH 64 /* D-Bus path */
 #define NETWORK_PATH "/org/bluez/network"
 
-int network_init(DBusConnection *conn);
+struct network_conf {
+	gboolean connection_enabled;
+	gboolean server_enabled;
+	gboolean security;
+	char *iface_prefix;
+	char *panu_script;
+	char *gn_script;
+	char *nap_script;
+	char *gn_iface;
+	char *nap_iface;
+};
+
+int network_init(DBusConnection *conn, struct network_conf *service_conf);
 void network_exit(void);
 
 int network_del_stored_info(bdaddr_t *src, uint16_t uuid);
