@@ -59,7 +59,6 @@
 #include "bridge.h"
 #include "common.h"
 #include "manager.h"
-#include "server.h"
 
 /* Pending Authorization */
 struct setup_session {
@@ -74,22 +73,22 @@ struct setup_session {
 
 /* Main server structure */
 struct network_server {
-	bdaddr_t		src;		/* Bluetooth Local Address */
-	char			*iface;		/* Routing interface */
-	char			*name;		/* Server service name */
-	char			*range;		/* IP Address range */
-	char			*path;		/* D-Bus path */
-	gboolean		enable;		/* Enable flag*/
-	gboolean		secure;		/* Security flag*/
-	uint32_t		record_id;	/* Service record id */
-	uint16_t		id;		/* Service class identifier */
-	GSList			*clients;	/* Active connections */
+	bdaddr_t	src;		/* Bluetooth Local Address */
+	char		*iface;		/* Routing interface */
+	char		*name;		/* Server service name */
+	char		*range;		/* IP Address range */
+	char		*path;		/* D-Bus path */
+	gboolean	enable;		/* Enable flag */
+	uint32_t	record_id;	/* Service record id */
+	uint16_t	id;		/* Service class identifier */
+	GSList		*clients;	/* Active connections */
 };
 
-static char netdev[16] = "bnep%d";
 static GIOChannel *bnep_io = NULL;
 static DBusConnection *connection = NULL;
 static GSList *setup_sessions = NULL;
+static const char *prefix = NULL;
+static gboolean security = TRUE;
 
 static int store_property(bdaddr_t *src, uint16_t id,
 			const char *key, const char *value)
@@ -134,7 +133,7 @@ static void add_lang_attr(sdp_record_t *r)
 }
 
 static int create_server_record(sdp_buf_t *buf, const char *name,
-					uint16_t id, dbus_bool_t secure)
+					uint16_t id)
 {
 	sdp_list_t *svclass, *pfseq, *apseq, *root, *aproto;
 	uuid_t root_uuid, pan, l2cap, bnep;
@@ -142,7 +141,7 @@ static int create_server_record(sdp_buf_t *buf, const char *name,
 	sdp_list_t *proto[2];
 	sdp_data_t *v, *p;
 	uint16_t psm = BNEP_PSM, version = 0x0100;
-	uint16_t security_desc = (secure ? 0x0001 : 0x0000);
+	uint16_t security_desc = (security ? 0x0001 : 0x0000);
 	uint16_t net_access_type = 0xfffe;
 	uint32_t max_net_access_rate = 0;
 	const char *desc = "BlueZ PAN service";
@@ -178,7 +177,7 @@ static int create_server_record(sdp_buf_t *buf, const char *name,
 		profile[0].version = 0x0100;
 		pfseq = sdp_list_append(NULL, &profile[0]);
 		sdp_set_profile_descs(&record, pfseq);
-		
+
 		sdp_set_info_attr(&record, name, NULL, desc);
 		break;
 	case BNEP_SVC_PANU:
@@ -303,6 +302,7 @@ static void authorization_callback(DBusPendingCall *pcall, void *data)
 	char path[MAX_PATH_LENGTH], devname[16];
 	uint16_t response = BNEP_CONN_NOT_ALLOWED;
 	DBusError derr;
+	const char *bridge;
 
 	if (!g_slist_find(setup_sessions, s)) {
 		dbus_message_unref(reply);
@@ -328,7 +328,7 @@ static void authorization_callback(DBusPendingCall *pcall, void *data)
 	}
 
 	memset(devname, 0, 16);
-	strncpy(devname, netdev, 16);
+	strncpy(devname, prefix, strlen(prefix));
 
 	if (bnep_connadd(s->nsk, s->dst_role, devname) < 0)
 		goto failed;
@@ -336,19 +336,20 @@ static void authorization_callback(DBusPendingCall *pcall, void *data)
 	info("Authorization succedded. New connection: %s", devname);
 	response = BNEP_SUCCESS;
 
-	if (bridge_add_interface("pan0", devname) < 0) {
-		error("Can't add %s to the bridge: %s(%d)",
-				devname, strerror(errno), errno);
-		goto failed;
-	}
+	bridge = bridge_get_name(ns->id);
+	if (bridge) {
+		if (bridge_add_interface(ns->id, devname) < 0) {
+			error("Can't add %s to the bridge: %s(%d)",
+					devname, strerror(errno), errno);
+			goto failed;
+		}
 
-	bnep_if_up(devname, TRUE);
-	bnep_if_up("pan0", TRUE);
+		bnep_if_up(devname, 0);
+	} else
+		bnep_if_up(devname, ns->id);
 
 	ns->clients = g_slist_append(ns->clients, g_strdup(s->address));
 
-	/* FIXME: Enable routing if applied */
-
 	/* FIXME: send the D-Bus message to notify the new bnep iface */
 
 failed:
@@ -593,7 +594,8 @@ static gboolean connect_event(GIOChannel *chan,
 	return TRUE;
 }
 
-int server_init(DBusConnection *conn)
+int server_init(DBusConnection *conn, const char *iface_prefix,
+		gboolean secure)
 {
 	struct l2cap_options l2o;
 	struct sockaddr_l2 l2a;
@@ -638,14 +640,14 @@ int server_init(DBusConnection *conn)
 		goto fail;
 	}
 
-	/* FIXME: Set link mode - it is applied to all servers */
-	lm = 0;
+	lm = secure? L2CAP_LM_SECURE : 0;
 	if (lm && setsockopt(sk, SOL_L2CAP, L2CAP_LM, &lm, sizeof(lm)) < 0) {
 		err = errno;
 		error("Failed to set link mode. %s(%d)",
 					strerror(err), err);
 		goto fail;
 	}
+	security = secure;
 
 	if (listen(sk, 1) < 0) {
 		err = errno;
@@ -654,6 +656,7 @@ int server_init(DBusConnection *conn)
 	}
 
 	connection = dbus_connection_ref(conn);
+	prefix = iface_prefix;
 
 	bnep_io = g_io_channel_unix_new(sk);
 	g_io_channel_set_close_on_unref(bnep_io, FALSE);
@@ -699,7 +702,7 @@ static uint32_t add_server_record(struct network_server *ns)
 		return 0;
 	}
 
-	if (create_server_record(&buf, ns->name, ns->id, ns->secure) < 0) {
+	if (create_server_record(&buf, ns->name, ns->id) < 0) {
 		error("Unable to allocate new service record");
 		dbus_message_unref(msg);
 		return 0;
@@ -751,7 +754,7 @@ static int update_server_record(struct network_server *ns)
 		return -ENOMEM;
 	}
 
-	if (create_server_record(&buf, ns->name, ns->id, ns->secure) < 0) {
+	if (create_server_record(&buf, ns->name, ns->id) < 0) {
 		error("Unable to allocate new service record");
 		dbus_message_unref(msg);
 		return -1;
@@ -915,6 +918,22 @@ static DBusHandlerResult disable(DBusConnection *conn,
 	return send_message_and_unref(conn, reply);
 }
 
+static DBusHandlerResult is_enabled(DBusConnection *conn, DBusMessage *msg,
+					void *data)
+{
+	struct network_server *ns = data;
+	DBusMessage *reply;
+
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+	dbus_message_append_args(reply, DBUS_TYPE_BOOLEAN, &ns->enable,
+					DBUS_TYPE_INVALID);
+
+	return send_message_and_unref(conn, reply);
+}
+
 static DBusHandlerResult set_name(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -1013,58 +1032,6 @@ static DBusHandlerResult set_routing(DBusConnection *conn, DBusMessage *msg,
 	return send_message_and_unref(conn, reply);
 }
 
-static DBusHandlerResult set_security(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct network_server *ns = data;
-	DBusMessage *reply;
-	DBusError derr;
-	dbus_bool_t secure;
-
-	reply = dbus_message_new_method_return(msg);
-	if (!reply)
-		return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-	dbus_error_init(&derr);
-	if (!dbus_message_get_args(msg, &derr,
-				DBUS_TYPE_BOOLEAN, &secure,
-				DBUS_TYPE_INVALID)) {
-		err_invalid_args(conn, msg, derr.message);
-		dbus_error_free(&derr);
-		return DBUS_HANDLER_RESULT_HANDLED;
-	}
-
-	ns->secure = secure;
-	if (ns->enable) {
-		if (update_server_record(ns) < 0) {
-			dbus_message_unref(reply);
-			return err_failed(conn, msg,
-				"Service record attribute update failed");
-		}
-	}
-
-	store_property(&ns->src, ns->id, "secure", "1");
-
-	return send_message_and_unref(conn, reply);
-}
-
-static DBusHandlerResult get_security(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct network_server *ns = data;
-	DBusMessage *reply;
-
-	reply = dbus_message_new_method_return(msg);
-	if (!reply)
-		return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-	dbus_message_append_args(reply,
-			DBUS_TYPE_BOOLEAN, &ns->secure,
-			DBUS_TYPE_INVALID);
-
-	return send_message_and_unref(conn, reply);
-}
-
 static DBusHandlerResult get_info(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -1139,13 +1106,12 @@ static DBusMethodVTable server_methods[] = {
 	{ "GetUUID",		get_uuid,		"",	"s"	},
 	{ "Enable",		enable,			"",	""	},
 	{ "Disable",		disable,		"",	""	},
+	{ "IsEnabled",		is_enabled,		"",	"b"	},
 	{ "SetName",		set_name,		"s",	""	},
 	{ "GetName",		get_name,		"",	"s"	},
 	{ "SetAddressRange",	set_address_range,	"ss",	""	},
 	{ "SetRouting",		set_routing,		"s",	""	},
-	{ "SetSecurity",	set_security,		"b",	""	},
-	{ "GetSecurity",	get_security,		"",	"b"	},
-	{ "GetInfo",		get_info,		"",	"{sv}"	},
+	{ "GetInfo",		get_info,		"",	"a{sv}"	},
 	{ NULL, NULL, NULL, NULL }
 };
 
@@ -1216,14 +1182,6 @@ int server_register_from_file(const char *path, const bdaddr_t *src,
 		return -1;
 	}
 
-	ns->secure = FALSE;
-	str = textfile_get(filename, "secure");
-	if (str) {
-		if (strcmp("1", str) == 0)
-			ns->secure = TRUE;
-		g_free(str);
-	}
-
 	ns->range = textfile_get(filename, "address_range");
 	ns->iface = textfile_get(filename, "routing");
 
@@ -1265,8 +1223,10 @@ int server_store(const char *path)
 	char addr[18];
 
 	if (!dbus_connection_get_object_user_data(connection,
-				path, (void *) &ns))
+				path, (void *) &ns)) {
+		error("Unable to salve %s on storage", path);
 		return -ENOENT;
+	}
 
 	ba2str(&ns->src, addr);
 	if (ns->id == BNEP_SVC_NAP)
@@ -1286,8 +1246,6 @@ int server_store(const char *path)
 	if (ns->range)
 		textfile_put(filename, "range", ns->range);
 
-	textfile_put(filename, "secure", ns->secure ? "1": "0");
-
 	textfile_put(filename, "enabled", ns->enable ? "1": "0");
 
 	return 0;
