@@ -20,7 +20,9 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
-int server_init(DBusConnection *conn);
+
+int server_init(DBusConnection *conn, const char *iface_prefix,
+		gboolean secure);
 void server_exit();
 int server_register(const char *path, bdaddr_t *src, uint16_t id);
 int server_register_from_file(const char *path, const bdaddr_t *src,
