@@ -449,6 +449,8 @@ static int rfcomm_connect(struct pending_connect *pc)
 		if (!(errno == EAGAIN || errno == EINPROGRESS)) {
 			error("connect() failed: %s (%d)",
 					strerror(errno), errno);
+			g_io_channel_unref(pc->io);
+			pc->io = NULL;
 			goto fail;
 		}
 
@@ -559,10 +561,12 @@ static void record_reply(DBusPendingCall *call, void *data)
 		}
 
 		port_store(&pc->src, &dst, err, ch, svcname);
+
+		port_register(pc->conn, err, &pc->src, &dst, port_name,
+			      path, svcname);
 		if (svcname)
 			g_free(svcname);
 
-		port_register(pc->conn, err, &dst, port_name, path);
 		ports_paths = g_slist_append(ports_paths, g_strdup(path));
 
 		reply = dbus_message_new_method_return(pc->msg);
@@ -612,6 +616,12 @@ static int get_record(struct pending_connect *pc, uint32_t handle,
 			DBUS_TYPE_UINT32, &handle,
 			DBUS_TYPE_INVALID);
 
+	/* Unref get_handles pending call */
+	if (pc->pcall) {
+		dbus_pending_call_unref(pc->pcall);
+		pc->pcall = NULL;
+	}
+
 	if (!dbus_connection_send_with_reply(pc->conn, msg, &pc->pcall, -1)) {
 		error("Can't send D-Bus message.");
 		dbus_message_unref(msg);
@@ -840,7 +850,7 @@ static DBusHandlerResult create_port(DBusConnection *conn,
 
 	snprintf(port_name, sizeof(port_name), "/dev/rfcomm%d", err);
 	port_store(&src, &dst, err, val, NULL);
-	port_register(conn, err, &dst, port_name, path);
+	port_register(conn, err, &src, &dst, port_name, path, NULL);
 	ports_paths = g_slist_append(ports_paths, g_strdup(path));
 
 	reply = dbus_message_new_method_return(msg);
@@ -897,6 +907,7 @@ static DBusHandlerResult remove_port(DBusConnection *conn,
 	struct rfcomm_dev_info di;
 	DBusError derr;
 	const char *path;
+	GSList *l;
 	int16_t id;
 
 	dbus_error_init(&derr);
@@ -927,6 +938,12 @@ static DBusHandlerResult remove_port(DBusConnection *conn,
 			DBUS_TYPE_STRING, &path,
 			DBUS_TYPE_INVALID);
 
+	l = g_slist_find_custom(ports_paths, path, (GCompareFunc) strcmp);
+	if (l) {
+		g_free(l->data);
+		ports_paths = g_slist_remove(ports_paths, l->data);
+	}
+
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
 
@@ -1614,7 +1631,7 @@ static DBusHandlerResult proxy_set_serial_params(DBusConnection *conn,
 static DBusMethodVTable proxy_methods[] = {
 	{ "Enable",			proxy_enable,			"",	""	},
 	{ "Disable",			proxy_disable,			"",	""	},
-	{ "GetInfo",			proxy_get_info,			"",	"{sv}"	},
+	{ "GetInfo",			proxy_get_info,			"",	"a{sv}"	},
 	{ "SetSerialParameters",	proxy_set_serial_params, 	"syys",	""	},
 	{ NULL, NULL, NULL, NULL },
 };
@@ -2137,7 +2154,7 @@ static DBusSignalVTable manager_signals[] = {
 
 static void parse_port(char *key, char *value, void *data)
 {
-	char path[MAX_PATH_LENGTH], port_name[16], dst_addr[18];
+	char path[MAX_PATH_LENGTH], port_name[16], dst_addr[18], *svc;
 	char *src_addr = data;
 	bdaddr_t dst, src;
 	int ch, id;
@@ -2149,6 +2166,10 @@ static void parse_port(char *key, char *value, void *data)
 	if (sscanf(value,"%d:", &ch) != 1)
 		return;
 
+	svc = strchr(value, ':');
+	if (svc && *svc)
+		svc++;
+
 	str2ba(dst_addr, &dst);
 	str2ba(src_addr, &src);
 
@@ -2157,7 +2178,8 @@ static void parse_port(char *key, char *value, void *data)
 
 	snprintf(port_name, sizeof(port_name), "/dev/rfcomm%d", id);
 
-	if (port_register(connection, id, &dst, port_name, path) < 0) {
+	if (port_register(connection, id, &src, &dst,
+				port_name, path, svc) < 0) {
 		rfcomm_release(id);
 		return;
 	}
